module Spec
  module Rails
    module DSL
      module ControllerBehaviourHelpers
        class << self
          def included(mod)
            mod.send :include, ExampleMethods
            mod.send :extend,  BehaviourMethods
          end
        end
        
        module BehaviourMethods
          attr_accessor :controller_class_name # :nodoc:
          
          # Use this to instruct RSpec to render views in your controller examples (Integration Mode).
          # 
          #   context "ThingController" do
          #     integrate_views
          #     ...
          #
          # See Spec::Rails::DSL::ControllerBehaviour for more information about
          # Integration and Isolation modes.
          def integrate_views
            @integrate_views = true
          end
          def integrate_views? # :nodoc:
            @integrate_views
          end
        end
        
        module ExampleMethods
          # Uses ActionController::Routing::Routes to generate
          # the correct route for a given set of options.
          # == Examples
          #   route_for(:controller => 'registrations', :action => 'edit', :id => 1)
          #     => '/registrations/1;edit'
          def route_for(options)
            ensure_that_routes_are_loaded
            routes = ActionController::Routing::Routes.generate(options)
            # Rails 1.1.6
            return routes[0] if routes.is_a?(Array)
            # Rails 1.2
            return routes if routes.is_a?(String)
          end

          #backwards compatibility to RSpec 0.7.0-0.7.3
          alias_method :routing, :route_for
        end
      end

      module ControllerInstanceMethods #:nodoc:

        # === render(options = nil, deprecated_status = nil, &block)
        #
        # This gets added to the controller's singleton meta class,
        # allowing Controller Specs to run in two modes, freely switching
        # from context to context.
        def render(options=nil, deprecated_status=nil, &block)
          unless block_given?
            unless integrate_views?
              @template.stub!(:file_exists?).and_return(true)
              @template.should_receive(:render_template).any_number_of_times.and_return(true) { |*args|
                @first_render ||= args[2]
              }
              @template.stub!(:find_template_extension_for).and_return("rhtml")
              @template.stub!(:read_template_file).and_return("this is fake content generated by rspec")
            end
          end
          response.controller_path = controller_path
          
          super unless performed?
        end
      
        def response(&block)
          #NOTE - we're setting @update for the assert_select_spec - kinda weird, huh?
          # Should be_gone('after 0.9')
          @update = block
          @_response || @response
        end
      
        def integrate_views!
          @integrate_views = true
        end

      private

        def integrate_views?
          @integrate_views
        end
      end
    
      # The methods provided by Spec::Rails::DSL::ControllerEvalContext
      # are available to you in Controller Specs.
      #
      # The Public Class Methods are to be used within the +context+ block:
      #
      #   context "ThingController" do
      #     # public class methods go here
      #
      # The Public Instance Methods are to be used within the +specify+ block:
      #
      #   context "ThingController" do
      #     specify "should do stuff" do
      #       # public instance methods go here
      #
      # See Spec::Rails::DSL::ControllerBehaviour for more general information
      # on Controller Specs
      class ControllerEvalContext < Spec::Rails::DSL::FunctionalEvalContext
        include Spec::Rails::DSL::ControllerBehaviourHelpers
        attr_reader :response, :request, :controller
      
        def setup #:nodoc:
          super
          
          # Some Rails apps explicitly disable ActionMailer in environment.rb
          if defined?(ActionMailer)
            @deliveries = []
            ActionMailer::Base.deliveries = @deliveries
          end

          unless @controller.class.ancestors.include?(ActionController::Base)
            Spec::Expectations.fail_with <<-EOE
  You have to declare the controller name in controller specs. For example:
    describe "The ExampleController" do
      controller_name "example" #invokes the ExampleController
    end
  EOE
          end
          @controller.metaclass.class_eval do
            # Rails 1.1.6 doesn't have controller_path, but >= 1.2.0 does
            unless instance_methods.include?("controller_path")
              def controller_path #:nodoc:
                self.class.name.underscore.gsub('_controller','')
              end
            end
            include ControllerInstanceMethods
          end
          @controller.integrate_views! if @integrate_views
          @controller.session = session rescue nil        end

      
        private
          def ensure_that_routes_are_loaded
            ActionController::Routing::Routes.reload if ActionController::Routing::Routes.empty?
          end
      end

      # Controller Specs live in $RAILS_ROOT/spec/controllers/.
      #
      # Controller Specs use Spec::Rails::DSL::ControllerBehaviour, which supports running specs for
      # Controllers in two modes, which represent the tension between the more granular
      # testing common in TDD and the more high level testing built into
      # rails. BDD sits somewhere in between: we want to a balance between
      # specs that are close enough to the code to enable quick fault
      # isolation and far enough away from the code to enable refactoring
      # with minimal changes to the existing specs.
      #
      # == Isolation mode (default)
      #
      # No dependencies on views because none are ever rendered. The
      # benefit of this mode is that can spec the controller completely
      # independent of the view, allowing that responsibility to be
      # handled later, or by somebody else. Combined w/ separate view
      # specs, this also provides better fault isolation.
      #
      # == Integration mode
      #
      # To run in this mode, include the +integrate_views+ declaration
      # in your controller context:
      # 
      #   context "ThingController" do
      #     integrate_views
      #     ...
      # 
      # In this mode, controller specs are run in the same way that
      # rails functional tests run - one set of tests for both the
      # controllers and the views. The benefit of this approach is that
      # you get wider coverage from each spec. Experienced rails
      # developers may find this an easier approach to begin with, however
      # we encourage you to explore using the isolation mode and revel
      # in its benefits.
      #
      # See Spec::Rails::DSL::ControllerEvalContext for information
      # about methods you can use in your Controller Specs
      class ControllerBehaviour < Spec::DSL::Behaviour

        def execution_context(example=nil) # :nodoc:
          instance = execution_context_class.new(example)
          if controller_class_name.nil? && !described_type.nil?
            controller_klass_name = described_type.to_s
          else
            controller_klass_name = controller_class_name.to_s
          end
          integrate_views = integrate_views?
          instance.instance_eval {
            @controller_class_name = controller_klass_name
            @integrate_views = integrate_views
          }
          instance
        end

        def before_eval # :nodoc:
          inherit Spec::Rails::DSL::ControllerEvalContext
          configure
        end

      end
    end
  end
end